#ifndef IP_COLORCUBE_INC
#define IP_COLORCUBE_INC

#include "ipCore.h"

namespace Torch {

/** This class is designed to filter an image using a YUV colorcube

    @author Sebastien Marcel (marcel@idiap.ch)
    @version 2.0
    \Date
    @since 2.0
*/
class ipColorCube : public ipCore
{
   	// width of the input image
	int width;

	// height of the input image
	int height;

	// the colorcube
	unsigned char *cube;

	// internal variables
  	int ShiftUV;
  	int ShiftY;
  	int MY;
  	int MUV;

public:

	/// dimension of Y component
	int DimY;

	/// dimension of U and V components
  	int DimUV;

	/// the binary image of filtered pixels (in bytes)
	unsigned char *binaryimage;

	/// the binary image of filtered pixels (in real)
	real *realbinaryimage;

	/// number of filtered pixels
	int n_pixels_filtered;

	/// pixel value of the background
	real background;
	
	//-----
	
	/** constructor
	    
	    @param width_ is the #width# of the input image
	    @param height_ is the #height# of the input image
	    @param coding_ is the #coding# format of the input image
	    @param background_ pixel value of the background
	*/
    	ipColorCube(int width_, int height_, const char *coding_, real background_ = 120.0);

	///
	virtual ~ipColorCube();

	///
	virtual void process(Sequence *seq_in);

	//-----
	
	/// load a colorcube
	virtual void loadXFile(XFile *file);

	/// save a colorcube
	virtual void saveXFile(XFile *file);

	//-----

	/// get a element of the colorcube
	unsigned char *getcube(unsigned char y, unsigned char u, unsigned char v);
	
	/// add a color in the colocube
	void add(int n_inputs_, real *data_);
	
	/// remove a color from the colorcube
	void remove(int n_inputs_, real *data_);

};

}

#endif
