#ifndef IPHISTOEQUAL_INC
#define IPHISTOEQUAL_INC

#include "ipCore.h"

namespace Torch {

/** This class is designed to enhance an image using histogram equalisation

    \begin{verbatim}

         +---+          +--------------+         +---+
         |xxx|	        |              |         |XXX|
	 |xxx|   ---->  | ipHistoEqual | ---->   |XXX|
         |xxx|          |              |         |XXX|
         +---+          +--------------+         +---+

    \end{verbatim}
    
    @author Sebastien Marcel (marcel@idiap.ch)
    @version 2.0
    \Date
    @since 2.0
*/
class ipHistoEqual : public ipCore
{
   	// width of the input image
	int width;

	// height of the input image
	int height;
	
	//
   	int histo_size;

	//
   	real *histogram;

	//
   	real *cumulhistogram;
	
public:
	/** constructor
	    
	    @param width_ is the #width# of the input image
	    @param height_ is the #height# of the input image
	    @param coding_ is the #coding# format of the input image
	*/
    	ipHistoEqual(int width_, int height_, const char *coding_);

	///
	virtual ~ipHistoEqual();

	///
	void process(Sequence *seq_in);	
};

}

#endif
