#ifndef IPNORM_BEST_FIT_LINEAR_INC
#define IPNORM_BEST_FIT_LINEAR_INC

#include "ipCore.h"
#include "fastMatrix.h"

namespace Torch {

/** This class is designed to normalize an image by substracting best-fit linear plane

    \begin{verbatim}

         +---+          +---------------------+         +---+
         |...|	        |                     |         |000|
	 |..o|   ---->  | ipNormBestFitLinear | ---->   |000|
         |.oo|          |                     |         |000|
         +---+          +---------------------+         +---+

	 image                                   image 
	 
    \end{verbatim}
    
    @author Sebastien Marcel (marcel@idiap.ch)
    @version 2.0
    \Date
    @since 2.0
*/
class ipNormBestFitLinear : public ipCore
{
   	// width of the input image
	int width;

	// height of the input image
	int height;
	
	// fast matrix manupulation
	fastMatrix *mat;
	
	// internal vector and matrices
	real *X;
	real *U;
	real *W;
	real **A;
	real **A_1;
	real **Macc;
	
public:
	/** constructor
	    
	    @param width_ is the #width# of the input image
	    @param height_ is the #height# of the input image
	    @param coding_ is the #coding# format of the input image
	*/
    	ipNormBestFitLinear(int width_, int height_, const char *coding_);

	///
	virtual ~ipNormBestFitLinear();

	///
	virtual void process(Sequence *seq_in);
};

}

#endif
