#ifndef IPSCALE_YX_INC
#define IPSCALE_YX_INC

#include "ipCore.h"

namespace Torch {

/** This class is designed to scale an image

    \begin{verbatim}

         +----+          +--------------+         
         |XXXX|	         |              |         +--+
	 |XXXX|   ---->  |  ipScale_yx  | ---->   |XX|
         |XXXX|          |              |         |XX|
         +----+          +--------------+         +--+

	 image                                 scaled image 
	 
    \end{verbatim}
    
    @author Sebastien Marcel (marcel@idiap.ch)
    @version 2.0
    \Date
    @since 2.0
*/
class ipScale_yx : public ipCore
{
   	// width of the input image
	int width_in;

	// height of the input image
	int height_in;

	// width of the output image
	int width_out;

	// height of the output image
	int height_out;

	// internal variables
	int size_buffer;
	real *buffer;
	
public:
	/** constructor
	
	    @param width_in_ is the #width# of the input image
	    @param height_in_ is the #height# of the input image
	    @param coding_ is the #coding# format of the input image
	    @param width_out_ is the #width# of the output image
	    @param height_out_ is the #height# of the output image
	*/
    	ipScale_yx(int width_in_, int height_in_, char *coding_, int width_out_, int height_out_);

	///
	virtual ~ipScale_yx();

	///
	virtual void process(Sequence *seq_in);

	//-----
	
	///
        virtual void setInputSize(int width, int height);

	///
	virtual int getWidthOut(void);

	///
        virtual int getHeightOut(void);

	///
        virtual void setOutputSize(int width, int height);

};

int YX_scale_gray(int width_src, int height_src, real *src_, int width_dst, int height_dst, real *dst_, real *buffer);
int YX_scale_rgb(int width_src, int height_src, real *src_, int width_dst, int height_dst, real *dst_, real *buffer);

}

#endif
