#include "ipShift.h"

namespace Torch {

ipShift::ipShift(int width_, int height_, const char *coding_) : ipCore(coding_)
{
   	width = width_;
        height = height_;
	
	addIOption("shiftx", &x_shift, 0, "x shift");
	addIOption("shifty", &y_shift, 0, "y shift");

	seq_out = new(allocator) Sequence(1, width_*height_*n_planes);
}

void ipShift::process(Sequence *seq_in)
{
	int w_;
	int h_;
	int offset_in;
	int offset_out;
	
	switch(coding_id)
	{
	case 0:
		for(int h = 0; h < height; h++)
		{
			h_ = h + y_shift;

			for(int w = 0; w < width; w++)
			{
				w_ = w + x_shift;

				if(((h_ >= 0) && (h_ < height)) && ((w_ >= 0) && (w_ < width)))
				{
				   	offset_out = w_ + h_ * width;
					offset_in = w + h * width;
				
				 	seq_out->frames[0][offset_out] = seq_in->frames[0][offset_in];
				}
			}
		}
		break;
	case 1:
	case 2:
		for(int h = 0; h < height; h++)
		{
			h_ = h + y_shift;

			for(int w = 0; w < width; w++)
			{
				w_ = w + x_shift;

				if(((h_ >= 0) && (h_ < height)) && ((w_ >= 0) && (w_ < width)))
				{
				   	offset_out = 3 * (w_ + h_ * width);
					offset_in = 3 * (w + h * width);
				
				 	seq_out->frames[0][offset_out] = seq_in->frames[0][offset_in];
				 	seq_out->frames[0][offset_out+1] = seq_in->frames[0][offset_in+1];
				 	seq_out->frames[0][offset_out+2] = seq_in->frames[0][offset_in+2];
				}
			}
		}
		break;
	case 3:
		message("ipShift::process() not implemented for float format");
		break;
	}
}

ipShift::~ipShift()
{
}

}

