#include "ipStat.h"

namespace Torch {

ipStat::ipStat(int width_, int height_, const char *coding_) : ipCore(coding_)
{
	width = width_;
        height = height_;

	seq_out = new(allocator) Sequence(n_planes, 2);
}

void ipStat::process(Sequence *seq_in)
{
	int input_size = width * height;

	real mean0 = 0.0;
	real stdv0 = 0.0;
	real mean1 = 0.0;
	real stdv1 = 0.0;
	real mean2 = 0.0;
	real stdv2 = 0.0;
	
	switch(coding_id)
	{
	case 0:
		for(int i = 0 ; i < input_size ; i++)
		{	
		   	real z = seq_in->frames[0][i];
	   		mean0 += z;
			stdv0 += z*z;
		}

		mean0 /= input_size;
		stdv0 /= input_size;
		stdv0 -= mean0*mean0;
		if(stdv0 <= 0)
		{
		   	if(verbose) warning("ipStat::process() null stdv replaced by 1");

		   	stdv0 = 1.0;
		}
		else stdv0 = sqrt(stdv0);
	
        	seq_out->frames[0][0] = mean0;
	        seq_out->frames[0][1] = stdv0;

		break;
	case 1:
	case 2:
		for(int i = 0 ; i < input_size ; i++)
		{	
		   	real z = seq_in->frames[0][3*i];
	   		mean0 += z;
			stdv0 += z*z;
			
		   	z = seq_in->frames[0][3*i+1];
	   		mean1 += z;
			stdv1 += z*z;
			
		   	z = seq_in->frames[0][3*i+2];
	   		mean2 += z;
			stdv2 += z*z;
		}

		mean0 /= input_size;
		stdv0 /= input_size;
		stdv0 -= mean0*mean0;
		if(stdv0 <= 0)
		{
		   	if(verbose) warning("ipStat::process() null stdv0 replaced by 1");

		   	stdv0 = 1.0;
		}
		else stdv0 = sqrt(stdv0);
	
		mean1 /= input_size;
		stdv1 /= input_size;
		stdv1 -= mean1*mean1;
		if(stdv1 <= 0)
		{
		   	if(verbose) warning("ipStat::process() null stdv1 replaced by 1");

		   	stdv1 = 1.0;
		}
		else stdv1 = sqrt(stdv1);
	
		mean2 /= input_size;
		stdv2 /= input_size;
		stdv2 -= mean2*mean2;
		if(stdv2 <= 0)
		{
		   	if(verbose) warning("ipStat::process() null stdv2 replaced by 1");

		   	stdv2 = 1.0;
		}
		else stdv2 = sqrt(stdv2);
	
        	seq_out->frames[0][0] = mean0;
	        seq_out->frames[0][1] = stdv0;

        	seq_out->frames[1][0] = mean1;
	        seq_out->frames[1][1] = stdv1;

        	seq_out->frames[2][0] = mean2;
	        seq_out->frames[2][1] = stdv2;

		break;
	case 3:
		message("ipStat not implemented for float format");
                break;
	}
}

ipStat::~ipStat()
{
}

}

