#include "jpegDiskXFile.h"

namespace Torch {

#ifdef HAVE_JPEG
typedef struct my_error_mgr *my_error_ptr;
#endif

jpegDiskXFile::jpegDiskXFile(const char *file_name, const char *open_flags) : ImageDiskXFile(file_name, open_flags)
{
   	n_planes = 3;
	coding = "rgb";
}

#ifdef HAVE_JPEG
METHODDEF(void) my_error_exit (j_common_ptr cinfo)
{
	/* cinfo->err really points to a my_error_mgr struct, so coerce
	   pointer */
        my_error_ptr myerr = (my_error_ptr) cinfo->err;

	/* Always display the message. */
	/* We could postpone this until after returning, if we
	   chose. */
	(*cinfo->err->output_message) (cinfo);
		           
	/* Return control to the setjmp point */
	longjmp(myerr->setjmp_buffer, 1);
}
#endif

bool jpegDiskXFile::readHeader(int *width_, int *height_)
{
#ifdef HAVE_JPEG
	//dinfo.err = jpeg_std_error(&jerr);
	dinfo.err = jpeg_std_error(&jerr.pub);
	jerr.pub.error_exit = my_error_exit;

	if(setjmp(jerr.setjmp_buffer)) return false;
	
	jpeg_create_decompress(&dinfo);
        jpeg_stdio_src(&dinfo, file);
        (void) jpeg_read_header(&dinfo, true);
        (void) jpeg_start_decompress(&dinfo);

        width = dinfo.output_width;
	height = dinfo.output_height;

	//
	if(pixmap != NULL) delete [] pixmap;
	pixmap = new unsigned char [3 * width * height];

	//
	row_stride = dinfo.output_width * dinfo.output_components;

	if(dinfo.output_components != 3)
	{
		//warning("jpegDiskXFile::readHeader(). The jpeg image is not a color image. Loading as grayscale.");

		if(dinfo.output_components != 1)
			error("This is not a gray level one !");
	}
	
	buffer = (*dinfo.mem->alloc_sarray) ((j_common_ptr) &dinfo, JPOOL_IMAGE, row_stride, 1);
	
	*width_ = width;
	*height_ = height;

	return true;
#else
	warning("jpegDiskXFile::readHeader(). JPEG format not supported.");

	*width_ = 0;
	*height_ = 0;

	return false;
#endif
}

int jpegDiskXFile::readPixmap()
{
#ifdef HAVE_JPEG
        JSAMPLE *line;

	int i = 0;
        while(dinfo.output_scanline < dinfo.output_height) 
        {
                (void) jpeg_read_scanlines(&dinfo, buffer, 1);

                line = buffer[0];
                int j = 0;

                while(j < row_stride) 
                {
		   	if(dinfo.output_components == 1)
			{
                        	pixmap[i++] = (unsigned char) line[j];
                        	pixmap[i++] = (unsigned char) line[j];
	                        pixmap[i++] = (unsigned char) line[j++];
			}
			else
			{
                        	pixmap[i++] = (unsigned char) line[j++];
                        	pixmap[i++] = (unsigned char) line[j++];
	                        pixmap[i++] = (unsigned char) line[j++];
			}
                }
        }

        (void) jpeg_finish_decompress(&dinfo);
        jpeg_destroy_decompress(&dinfo); 

	return 3*width*height;
#else
	warning("jpegDiskXFile::readPixmap(). JPEG format not supported.");

	return -1;
#endif
}

bool jpegDiskXFile::writeHeader(int width_, int height_)
{
#ifdef HAVE_JPEG
   	width = width_;
	height = height_;

	//cinfo.err = jpeg_std_error(&jerr);
	cinfo.err = jpeg_std_error(&jerr.pub);
	jerr.pub.error_exit = my_error_exit;

	if(setjmp(jerr.setjmp_buffer)) return false;

	jpeg_create_compress(&cinfo);
	jpeg_stdio_dest(&cinfo, file);
	
	int quality = 95;
  	jpeg_set_quality(&cinfo, quality, true);   

	cinfo.image_width = width; 	/* image width and height, in pixels */
  	cinfo.image_height = height;
  	cinfo.input_components = 3;	/* # of color components per pixel */
  	cinfo.in_color_space = JCS_RGB; /* colorspace of input image */
	cinfo.optimize_coding = 100;
        cinfo.smoothing_factor = 0;

  	jpeg_set_defaults(&cinfo);

	jpeg_start_compress(&cinfo, true);

  	row_stride = width * 3;
  
	return true;
#else
	warning("jpegDiskXFile::writeHeader(). JPEG format not supported.");

	return -1;
#endif
}

int jpegDiskXFile::writePixmap(unsigned char *pixmap_)
{
#ifdef HAVE_JPEG
   	JSAMPROW row_pointer[1];

   	while(cinfo.next_scanline < cinfo.image_height) 
	{
    		row_pointer[0] = &pixmap_[cinfo.next_scanline * row_stride];
    		(void) jpeg_write_scanlines(&cinfo, row_pointer, 1);
  	}

  	jpeg_finish_compress(&cinfo);
  	jpeg_destroy_compress(&cinfo);

	return 3*width*height;
#else
	warning("jpegDiskXFile::writePixmap(). JPEG format not supported.");

	return -1;
#endif
}

jpegDiskXFile::~jpegDiskXFile()
{
	if(pixmap != NULL) delete [] pixmap;
}

}
