#ifndef JPEG_DISK_X_FILE_INC
#define JPEG_DISK_X_FILE_INC

#include "ImageDiskXFile.h"

namespace Torch {

#ifdef HAVE_JPEG
extern "C"
{
#include <jpeglib.h>
#include <setjmp.h>
}

struct my_error_mgr
{
	struct jpeg_error_mgr pub;    /* "public" fields */
	jmp_buf setjmp_buffer;        /* for return to caller */
};

#endif

/** This class is designed to handle a JPEG image on the disk
    
    JPEG source code can be found \URL[here]{http://www.ijg.org/files/jpegsrc.v6b.tar.gz}.

    @author Sebastien Marcel (marcel@idiap.ch)
    @version 2.0
    \Date
    @since 1.0
*/
class jpegDiskXFile : public ImageDiskXFile
{
#ifdef HAVE_JPEG
	struct jpeg_decompress_struct dinfo;
	struct jpeg_compress_struct cinfo;
	struct my_error_mgr jerr;
   	int row_stride;
        JSAMPARRAY buffer;
#endif
public:
   	/**@name constructor and destructor */
        //@{
	/** Makes a ImageDiskXFile from an filename.
	
	    @param file_name is the filename of the image
	    @param open_flags is the opening mode of the file
	*/
	jpegDiskXFile(const char *file_name, const char *open_flags);

	/// Destructor
    	virtual ~jpegDiskXFile();
	//@}
	
   	/**@name read/write image header and image pixmap */
        //@{
	/** read the header from the file.
	
	    @param width_ is the width of the image read in the header
	    @param height_ is the height of the image read in the header
	*/
	virtual bool readHeader(int *width_, int *height_);

	/// read the #pixmap# from the file
	virtual int readPixmap();

	/** write the header from the file.
	
	    @param width_ is the width of the image written in the header
	    @param height_ is the height of the image written in the header
	*/
	virtual bool writeHeader(int width_, int height_);

	/// write the #pixmap_# into the file
	virtual int writePixmap(unsigned char *pixmap_);
	//@}
};

}

#endif
