const char *help = "\
progname: cmpbindata.cc\n\
code2html: This program compares two bindata files.\n\
version: Torch3 vision2.0, 2004-2005\n\
(c) Sebastien Marcel (marcel@idiap.ch)\n";

#include "DiskXFile.h"
#include "CmdLine.h"

using namespace Torch;

real MSE(int n_inputs, real *x, real *y)
{
	real d_ = 0.0;

	for(int i = 0 ; i < n_inputs ; i++)
	{
		real z = x[i] - y[i];
		d_ += z * z;
	}
	
	return d_ / (real) n_inputs;
}

int main(int argc, char *argv[])
{
	char *filename_in1;
	char *filename_in2;
	bool verbose;
	
	CmdLine cmd;
	cmd.setBOption("write log", false);

	cmd.info(help);
	cmd.addText("\nArguments:");
	cmd.addSCmdArg("filename in1", &filename_in1, "input bindata filename 1");
	cmd.addSCmdArg("filename in2", &filename_in2, "input bindata filename 2");
	cmd.addText("\nArguments:");
	cmd.addBCmdOption("-verbose", &verbose, false, "verbose");
	cmd.read(argc, argv);

	DiskXFile *file1 = NULL;
	DiskXFile *file2 = NULL;
  
	int n_patterns1;
	int n_inputs1;

	file1 = new DiskXFile(filename_in1, "r");
		
	file1->read(&n_patterns1, sizeof(int), 1);
	file1->read(&n_inputs1, sizeof(int), 1);

	if(verbose)
	{
		print("Reading bindata file (%s)\n", filename_in1);
		print("   n_inputs = %d\n", n_inputs1);
		print("   n_patterns = %d\n", n_patterns1);  
	}


	int n_patterns2;
	int n_inputs2;

	file2 = new DiskXFile(filename_in2, "r");
		
	file2->read(&n_patterns2, sizeof(int), 1);
	file2->read(&n_inputs2, sizeof(int), 1);

	if(verbose)
	{
		print("Reading bindata file (%s)\n", filename_in2);
		print("   n_inputs = %d\n", n_inputs2);
		print("   n_patterns = %d\n", n_patterns2);  
	}

	if(n_inputs1 != n_inputs2) error("n_inputs different");

	int n_inputs = n_inputs1;
	
	real **inputs1 = new real*[n_patterns1];
	for(int p = 0 ; p < n_patterns1 ; p++)
	{
		inputs1[p] = new real [n_inputs];
		file1->read(inputs1[p], sizeof(real), n_inputs);
	}

	real **inputs2 = new real*[n_patterns2];
	for(int p = 0 ; p < n_patterns2 ; p++)
	{
		inputs2[p] = new real [n_inputs];
		file2->read(inputs2[p], sizeof(real), n_inputs);
	}

	// Compare each pattern of file to patterns of file 1
	real mean = 0.0;

	for(int p2 = 0 ; p2 < n_patterns2 ; p2++)
	{
	   	real mean_ = 0.0;

		print("Pattern %d:\n", p2);
		for(int p1 = 0 ; p1 < n_patterns1 ; p1++)
		{
			real d_ = MSE(n_inputs, inputs1[p1], inputs2[p2]);

			print("   > %g\n", d_);

			mean_ += d_;
		}

		mean_ /= (real) n_patterns1;

		print("   mean = %g\n", mean_);

		mean += mean_;
	}
	
	mean /= (real) n_patterns2;

	print("Average distance between %s and %s:\n", filename_in1, filename_in2);
	print(" > %g\n", mean);

	for(int p = 0 ; p < n_patterns1 ; p++) delete [] inputs1[p];
	delete [] inputs1;

	for(int p = 0 ; p < n_patterns2 ; p++) delete [] inputs2[p];
	delete [] inputs2;

	delete file1;
	delete file2;

	return 0;
}
   
