const char *help = "\
progname: ppm2tiff.cc\n\
code2html: This program reads a ppm image and saves it as a tiff.\n\
version: Torch3 vision2.0, 2004-2005\n\
(c) Sebastien Marcel (marcel@idiap.ch)\n";

#include "ImageRgb.h"
#include "tiffDiskXFile.h"
#include "DiskXFile.h"
#include "CmdLine.h"

using namespace Torch;

int main(int argc, char **argv)
{
	char *image_filename;
	bool verbose;
  
	// Construct the command line
  	CmdLine cmd;
	cmd.setBOption("write log", false);
	
  	// Put the help line at the beginning
  	cmd.info(help);

  	cmd.addText("\nArguments:");
  	cmd.addSCmdArg("image filename", &image_filename, "image filename");
  	cmd.addText("\nOptions:");
  	cmd.addBCmdOption("-verbose", &verbose, false, "verbose");

	cmd.read(argc, argv);


  	tiffDiskXFile *tiff_file = NULL;
	Image *colorimage = NULL;

	colorimage = new ImageRgb();
	colorimage->setBOption("verbose", verbose);
	colorimage->load(image_filename);

	if(verbose)
	{
		print("Image info:\n");
		print("   width = %d\n", colorimage->width);
		print("   height = %d\n", colorimage->height);
		print("   format = %s (%d)\n", colorimage->coding, colorimage->n_planes);
	}

	tiff_file = new tiffDiskXFile("color.tiff", "w");
	tiff_file->writeHeader(colorimage->width, colorimage->height);
	tiff_file->writePixmap(colorimage->pixmap);
	delete tiff_file;

	delete colorimage;

	return(0);
}
