const char *help = "\
progname: ppmread.cc\n\
code2html: This program reads/writes a ppm image.\n\
version: Torch3 vision2.0, 2004-2005\n\
(c) Sebastien Marcel (marcel@idiap.ch)\n";

#include "ImageRgb.h"
#include "DiskXFile.h"
#include "CmdLine.h"

using namespace Torch;

int main(int argc, char **argv)
{
	char *image_filename;
	bool verbose;
  
	// Construct the command line
  	CmdLine cmd;
	cmd.setBOption("write log", false);
	
  	// Put the help line at the beginning
  	cmd.info(help);

  	cmd.addText("\nArguments:");
  	cmd.addSCmdArg("image filename", &image_filename, "image filename");
  	cmd.addText("\nOptions:");
  	cmd.addBCmdOption("-verbose", &verbose, false, "verbose");

	cmd.read(argc, argv);


  	DiskXFile *image_file = NULL;
	Image *image = NULL;

	image = new ImageRgb();
	image->setBOption("verbose", verbose);
	
	image_file = new DiskXFile(image_filename, "r");
	image->loadXFile(image_file);
	delete image_file;

	if(verbose)
	{
		print("Image info:\n");
		print("   width = %d\n", image->width);
		print("   height = %d\n", image->height);
		print("   format = %s (%d)\n", image->coding, image->n_planes);
	}

	image_file = new DiskXFile("test.ppm", "w");
	image->saveXFile(image_file);
	delete image_file;



	delete image;

	return(0);
}
