const char *help = "\
progname: shiftpgm.cc\n\
code2html: This program reads a pgm image and shift its content.\n\
version: Torch3 vision2.0, 2004-2005\n\
(c) Sebastien Marcel (marcel@idiap.ch)\n";

#include "ImageGray.h"
#include "ipShift.h"
#include "DiskXFile.h"
#include "CmdLine.h"

using namespace Torch;

int main(int argc, char **argv)
{
	char *image_filename;
	bool verbose;
	int shiftx, shifty;
  

  	CmdLine cmd;
	cmd.setBOption("write log", false);
  	cmd.info(help);
  	cmd.addText("\nArguments:");
  	cmd.addSCmdArg("image filename", &image_filename, "image filename");
  	cmd.addICmdArg("shift x", &shiftx, "shift x");
  	cmd.addICmdArg("shift y", &shifty, "shift y");
  	cmd.addText("\nOptions:");
  	cmd.addBCmdOption("-verbose", &verbose, false, "verbose");
	cmd.read(argc, argv);


  	DiskXFile *image_file = NULL;
	Image *image_in = NULL;
	Image *image_out = NULL;

	image_in = new ImageGray();
	image_in->setBOption("verbose", verbose);
	
	image_file = new DiskXFile(image_filename, "r");
	image_in->loadXFile(image_file);
	delete image_file;

	if(verbose)
	{
		print("Image info:\n");
		print("   width = %d\n", image_in->width);
		print("   height = %d\n", image_in->height);
		print("   format = %s (%d)\n", image_in->coding, image_in->n_planes);
	}

	ipCore *shift = NULL;

	shift = new ipShift(image_in->width, image_in->height, "gray");
	shift->setBOption("verbose", verbose);
	shift->setIOption("shiftx", shiftx);
	shift->setIOption("shifty", shifty);
	shift->process(image_in);
	
	image_out = new ImageGray();
	image_out->setBOption("verbose", verbose);

	image_out->copyFrom(image_in->width, image_in->height, shift->seq_out->frames[0], "gray");
	
	image_file = new DiskXFile("shift.pgm", "w");
	image_out->saveXFile(image_file);
	delete image_file;

	delete shift;
	delete image_out;
	delete image_in;

	return(0);
}
