#include "Rectangle2D.h"
#include "Matrix2D.h"

namespace Torch {

Rectangle2D::Rectangle2D(Point2D P0_, int w_, int h_)
{
	P0 = P0_;
	P1 = P0_;
	P2 = P0_;
	P3 = P0_;

	P1.x += w_;
	P2.x += w_;
	P2.y += h_;
	P3.y += h_;
}

Rectangle2D::Rectangle2D(int x_, int y_, int w_, int h_)
{
	P0.reset(x_, y_);
	P1.reset(x_, y_);
	P2.reset(x_, y_);
	P3.reset(x_, y_);

	P1.x += w_;
	P2.x += w_;
	P2.y += h_;
	P3.y += h_;
}

void Rectangle2D::reset(Point2D P0_, Point2D P1_, Point2D P2_, Point2D P3_)
{
	P0 = P0_;
	P1 = P1_;
	P2 = P2_;
	P3 = P3_;
}

void Rectangle2D::reset(Point2D P0_, int w_, int h_)
{
	P0 = P0_;
	P1 = P0_;
	P2 = P0_;
	P3 = P0_;

	P1.x += w_;
	P2.x += w_;
	P2.y += h_;
	P3.y += h_;
}
	
void Rectangle2D::reset(int x_, int y_, int w_, int h_)
{
	P0.reset(x_, y_);
	P1.reset(x_, y_);
	P2.reset(x_, y_);
	P3.reset(x_, y_);

	P1.x += w_;
	P2.x += w_;
	P2.y += h_;
	P3.y += h_;
}

void Rectangle2D::saveXFile(XFile *file)
{
   	P0.saveXFile(file);
   	P1.saveXFile(file);
   	P2.saveXFile(file);
}

void Rectangle2D::loadXFile(XFile *file)
{
   	P0.loadXFile(file);
   	P1.loadXFile(file);
   	P2.loadXFile(file);
}

const char *Rectangle2D::sprint()
{
	sprintf(buf_sprint, "[(%g, %g) (%g, %g) (%g, %g) (%g, %g)]", 
	      		P0.x, P0.y, 
			P1.x, P1.y,
			P2.x, P2.y,
			P3.x, P3.y);

	return buf_sprint;
}

void Rectangle2D::draw(Image *image, Color color)
{
	image->drawline(P0.x, P0.y, P1.x, P1.y, color);
	image->drawline(P1.x, P1.y, P2.x, P2.y, color);
	image->drawline(P2.x, P2.y, P3.x, P3.y, color);
	image->drawline(P3.x, P3.y, P0.x, P0.y, color);
}

#ifdef HAVE_X11
void Rectangle2D::xdraw(Display *pDisplay, Pixmap pixmap, GC gc, unsigned long color)
{
	XSetForeground(pDisplay, gc, color);

	XDrawLine(pDisplay, pixmap, gc, (int) P0.x, (int) P0.y, (int) P1.x, (int) P1.y);
	XDrawLine(pDisplay, pixmap, gc, (int) P1.x, (int) P1.y, (int) P2.x, (int) P2.y);
	XDrawLine(pDisplay, pixmap, gc, (int) P2.x, (int) P2.y, (int) P3.x, (int) P3.y);
	XDrawLine(pDisplay, pixmap, gc, (int) P3.x, (int) P3.y, (int) P0.x, (int) P0.y);
}
#endif

void Rectangle2D::fixI()
{
	P0.fixI();
	P1.fixI();
	P2.fixI();
	P3.fixI();
}

Rectangle2D operator*(Rectangle2D rs, Matrix2D m)
{
	Rectangle2D rd;
	
	rd.P0 = rs.P0 * m;
	rd.P1 = rs.P1 * m;
	rd.P2 = rs.P2 * m;
	rd.P3 = rs.P3 * m;

	return rd;
}

}
