#ifndef IP_SUBWINDOW_HAAR_INC
#define IP_SUBWINDOW_HAAR_INC

#include "ipSubWindow.h"
#include "ipHaar.h"
#include "Machine.h"

namespace Torch {

/** This class is designed to classify a subwindow using a cascade of haar-like classifiers

    This class should be used with an MultiScaleWindowScan.
	
    @author Sebastien Marcel (marcel@idiap.ch)
    @author Yann Rodriguez (yann.rodriguez@idiap.ch)
    @version 2.0
    \Date
    @since 1.0
*/
class ipSWhaar : public ipSubWindow
{
public:
	///
	Sequence *seq_input;

	///
	int *n_rejections_stage;
	
	/// integral image
	real *ii;

	/// integral image square
	real *ii2;

	/// all Haar-like masks
	ipHaar ***masks;

	/// all classifiers of the cascade
	Machine ***machines;

	/// number of stages in the cascade
	int n_stages;

	/// weights of each classifiers of the cascade
	real **weights;

	/// number of classifiers per stage
	int *n_trainers;

	/// thresholds for each stage
	real *thresholds;

	//-----
	
	///
	ipSWhaar(real *integralimage_,
		real *integralimage2_,
		int width_image_, int height_image_, 
		int width_pattern_, int height_pattern_, 
		const char *model_filename_, real threshold_,
		bool verbose = false,
		bool realstump = false);

	///
	virtual void process(Sequence *seq_in);

	///
	virtual void stat(void);
	
	///
	virtual ~ipSWhaar();
};

}

#endif
