#include "ipSWmlp.h"

namespace Torch {

ipSWmlp::ipSWmlp(int width_image_, int height_image_, 
		 int width_pattern_, int height_pattern_,
		 const char *model_filename_, real threshold_,
		 bool verbose_) 
		: ipSubWindow(width_image_, height_image_, width_pattern_, height_pattern_, threshold_)
{
   	verbose = verbose_;
	
	if(verbose)
	{
		print("ipSWmlp::ipSWmlp()\n");
		print("   width_image=%d\n", width_image);
		print("   height_image=%d\n", height_image);
		print("   width_pattern=%d\n", width_pattern);
		print("   height_pattern=%d\n", height_pattern);
	}

	size_pattern = width_pattern * height_pattern;
 
	sub_image = new(allocator) Sequence(1, size_pattern);	

	grayimage = new(allocator) ImageGray(width_pattern, height_pattern);

	
	/*************   load the model **************/
	mlp = new(allocator) MyMLP();
	mv_norm = new(allocator) MyMeanVarNorm(size_pattern, 1);
   
	mlp->load(model_filename_, mv_norm);
		
	if(verbose)
	{
		mlp->info();

		print("threshold: %g\n", threshold);
	}

	enhancing = new(allocator) ipHistoEqual(width_pattern, height_pattern, "float");
	smoothing = new(allocator) ipSmoothGaussian3(width_pattern, height_pattern, "gray", 0.25);
}

void ipSWmlp::process(Sequence *seq_in)
{
   	ispattern = false;
	
	cropGray(seq_in->frames[0], width_image, height_image, x_window, y_window, w_window, h_window, sub_image->frames[0]);

        enhancing->process(sub_image);
        smoothing->process(enhancing->seq_out);
	
	mv_norm->preProcessInputs(smoothing->seq_out);
	
	mlp->forward(smoothing->seq_out);
	   
	seq_out->frames[0][0] = -INF;

	if(mlp->outputs->frames[0][0] >= threshold)
	{
	   	ispattern = true;

		seq_out->frames[0][0] = mlp->outputs->frames[0][0];
		
		if(verbose)
			print(" -> mlp = %g\n", seq_out->frames[0][0]);

		if(verbose)
		{
			char str[255];

			sprintf(str, "subimage-%d-%d-%d-%d-%g.pgm", x_window, y_window, w_window, h_window, seq_out->frames[0][0]);

			grayimage->copyFrom(w_window, h_window, smoothing->seq_out->frames[0], "gray", 1.0);
			grayimage->save(str);
		}
	}
}

ipSWmlp::~ipSWmlp()
{
}

}

