#ifndef HAAR_FEATURE_MAKER_INC
#define HAAR_FEATURE_MAKER_INC

#include "ipHaar.h"

namespace Torch {

/** This class is designed to create a Haar-like feature vector

    It handles #n_masks_# kinds of Haar-like features of #max_size_# maximum
    size within an image of size #width_#x#height_#.The make function loops on
    all possible features and call the kernel function for each features. 

    @author Sebastien Marcel (marcel@idiap.ch)
    @author Yann Rodriguez (yann.rodriguez@idiap.ch)
    @version 2.0
    \Date
    @since 1.0
*/
class HaarFeatureMaker : public Object
{
private:
	/** if #create_# = false, the function only computes the total number of
		 possible features(size of the feature space). Otherwise, the function
		 "instanciates" the mask no #feature_id#, by calling the kernel() method.
	*/
	void computeFeatures(int w_step, int h_step, int mask_id_, bool create_);

public:
	/// number of different kind of masks (vertical, horizontal, diagonal, ...).
	int n_mask_id;

	/// size of the image where the features are defined in.
	int width;
	int height;

	/// min and max size of a feature width or height.
	int min_size;
	int max_size;

	/// size of the feature space
	int n_features;

	/// mask vector
	ipHaar **mask;

	
	///
	HaarFeatureMaker(int n_mask_id_, int width_, int height_, int min_size_, int max_size_);

	///-----

	virtual ~HaarFeatureMaker();	
};

}

#endif
