#ifndef VIDEO_SOURCE_INC
#define VIDEO_SOURCE_INC

#include "Object.h"
#include "VirtualPixmap.h"

namespace Torch {

#define VIDEOSOURCE_ERROR 0
#define VIDEOSOURCE_OK 1
#define VIDEOSOURCE_OPEN 2

/** This class is designed to handle a video source

    It opens/closes a VirtualPixmap and read a frame from it.

    @author Sebastien Marcel (marcel@idiap.ch) 
    @version 2.0
    \Date
    @since 1.0
*/
class VideoSource : public Object
{
protected:
   	/// internal status
  	short status;

public:
	/// verbose
	bool verbose;

	/// width of the image in the video source
  	int width; 

	/// height of the image in the video source
  	int height;

	//-----
	
	/// constructor
  	VideoSource(bool verbose_ = false);
  
	/// destructor
  	virtual ~VideoSource() {};

	/// initialize the video source
  	virtual bool initVideoSource() = 0;

	/// open the video source
  	virtual bool openVideoSource() = 0;

	/// close the video source
  	virtual bool closeVideoSource() = 0;

	/// read the current frame of the video source
  	virtual VirtualPixmap* getframe() = 0;

	/// release the current frame of the video source
  	virtual bool releaseframe(VirtualPixmap *) { return true; }; 

	/// return true if the video source is #OK# or #OPEN#
  	bool isOk();  
};
	
}

#endif
